.const
ENCODING_ANSI equ 0
ENCODING_UTF8 equ 1
ENCODING_UCS2BE equ 2
ENCODING_UCS2LE equ 3
;///BYTE ORDER MARKS:
;//they looks reversed as words
;//the original byte order is:
;//FE FF for UCS2BE
;//and
;//FF FE fo UCS2LE
UCS2BE_signature equ 0FFFEh
UCS2LE_signature equ  0FEFFh

.data
;///BYTE ORDER MARKS:
;///defined as byte in order to be inserted in text on saving:
UTF8_signature db 0EFh,0BBh,0BFh,0



.code
EncodeToUnicode PROC lpDocument:DWORD,lpnsize:LPDWORD,flags:DWORD ;//to unicode from <flag>
;//----------------------------------------------------------------//
;//The function gets the pointer to the opened file and            
;//a pointer to DWORD,that contains the size,                      
;//this parameter is LPDWORD ,so on exit, the variable in          
;//the calling procedure will be calculated,depending on conversion
;//EncodeDocument is used only on File Saving when encoding is	   
;//determined as Unicode BE and LE                                 
;//returning value is pointer to the converted string              
;//Is Dinamycally allocated and should be free in the              
;//calling procedure !                                             
;//I.E. - Swapping the two Buffers and sizes 
;//----------------------------------------------------------------//

	LOCAL lpDest:DWORD ;//the destination buffer
	LOCAL lpNoBOM :DWORD ;//the destination [buffer + 2]
	LOCAL wsize:DWORD ;//the unicode  size
	LOCAL nsize:DWORD ;//the multibyte size

		.if lpDocument == 0 || lpnsize == 0
			mov eax,FALSE
			ret
		.endif
		
		mov ebx,lpnsize
		mov eax,DWORD ptr[ebx]
		mov nsize,eax
		cdq
		mov ecx,2
		imul ecx
		mov wsize,eax ;//wsize = (nsize * 2)
		add eax,2 ;//space for BOM
		invoke SysAllocStringByteLen,0,eax
		mov lpDest,eax
		add eax,2
		mov lpNoBOM,eax
			
		invoke MultiByteToWideChar,SC_CP_UTF8,0,lpDocument,nsize,lpNoBOM,wsize
		
		;//free the old Buffer
		invoke SysFreeString,lpDocument
		;//swap only if UCS2BE:
		.if flags == ENCODING_UCS2BE
			invoke swapbytes,lpNoBOM,wsize
			mov ebx,lpDest
			mov word ptr[ebx],UCS2BE_signature
		.else
			mov ebx,lpDest
			mov word ptr[ebx],UCS2LE_signature
		.endif
		
		invoke lstrlenW,lpDest	
		cdq
		mov ecx,2
		imul ecx
		mov nsize,eax
		push nsize
		mov ebx,lpnsize
		pop dword ptr[ebx]
		mov eax,lpDest
		ret
ret
EncodeToUnicode ENDP


DecodeDocument PROC lpDocument:DWORD,lpwsize:LPDWORD,flags :DWORD
;//----------------------------------------------------------------
;//The function gets the pointer to the opened file and 
;//a pointer to DWORD,that contains the size,
;//this parameter is LPDWORD ,so on exit, the variable in
;//the calling procedure will be calculated,depending on conversion
;//DecodeDocument is used only on File Openning when encoding is
;//determined as Unicode BE and LE
;//returning value is pointer to the converted string
;//Is Dinamycally allocated and should be free in the
;//calling procedure !
;//I.E. - Swapping the two Buffers and sizes
;//----------------------------------------------------------------

	LOCAL lpSource:DWORD ;//temp pointer at the beginnig, and allocated buffer after that
	LOCAL lpDest:DWORD
	LOCAL wsize:DWORD
	LOCAL nsize:DWORD
		
		
		.if lpDocument == 0 || lpwsize == 0
			mov eax,FALSE
			ret
		.endif
		
		;//removing the signature
		mov ebx,lpDocument
		add ebx,2
		mov lpSource,ebx
		mov ebx,lpwsize
		mov eax, dword ptr[ebx]
		sub eax,1;//caused by removing the BOM it is substracted by one, because of the NULL terminator!
		mov nsize,eax 
		
		.if flags == ENCODING_UCS2BE ;BIG_ENDIAN
			invoke swapbytes,lpSource,nsize
		.endif
		
		;invoke lstrlenW,lpSource  and:
		;add eax,1 
		;mov wsize,eax ;//does the same job as below ,
		;//but this is faster way to examine the length:
		;mov ebx,lpwsize
		;mov eax, dword ptr[ebx]
		;cdq	
		;mov ecx,2
		;idiv ecx
		;mov wsize,eax
		;invoke SysAllocStringByteLen,0,nsize
		;mov lpDest,eax
		;/////Unfortunately, the second one does not do the job every time neither
		;////because opennig some Unicode files need a various number of 
		;///bytes to be allocated as Destination buffer
		;///That's why, i'm using the most secure way to examine the needed size:
		
		;///determine the needed space for allocating
		invoke WideCharToMultiByte,SC_CP_UTF8,0,lpSource,-1,0,0,0,0
		mov nsize,eax
		invoke SysAllocStringByteLen,0,eax
		mov lpDest,eax
		
		;//the 4th paramether bellow can be wsize either!
		invoke WideCharToMultiByte,SC_CP_UTF8,0,lpSource,-1,lpDest,nsize,0,0
		sub eax,1
		mov nsize,eax
		
		;//clearing the old buffer
		invoke SysFreeString,lpDocument
		
		;//NULL terminating
		mov ebx,lpDest
		add ebx,wsize
		mov byte ptr[ebx],NULL
		
		push nsize
		mov ebx,lpwsize
		pop dword ptr[ebx]
		mov eax,lpDest
		ret

DecodeDocument ENDP



swapbytes PROC PRIVATE lpBuffer:DWORD,bytesize:DWORD

	LOCAL wordsize:DWORD
		
		mov eax,bytesize
		cdq
		mov ecx,2
		idiv ecx
		mov wordsize,eax
		mov ebx,lpBuffer
		
		.while wordsize
			mov ax,WORD ptr[ebx]
			mov dx,ax
			shr dx,8
			or dh,al
			mov WORD ptr[ebx],dx 
			add ebx,2
			dec wordsize
		.endw

ret
swapbytes ENDP




OpenFileName PROC lpszFileName:DWORD

		LOCAL hFile :DWORD
		LOCAL lpBuffer	:DWORD
		LOCAL fsize :DWORD
		LOCAL readen:DWORD
		
			
			invoke CreateFile,lpszFileName,GENERIC_READ,FILE_SHARE_READ,0,
				OPEN_EXISTING,0,0
			mov hFile,eax
			
			.if eax != INVALID_HANDLE_VALUE
				invoke GetFileSize,hFile,NULL
				mov fsize,eax
				add eax,1
				invoke SysAllocStringByteLen,0,eax
				
				mov lpBuffer,eax
				invoke ReadFile,hFile,lpBuffer,fsize,ADDR readen,NULL
				invoke CloseHandle,hFile
				;/////encoding//////////////
				invoke DetermineEncoding,lpBuffer
				mov _encoding,eax
				.if eax == 1 ;UTF8
					;//removing signature so it will be not printed in hSci
					add lpBuffer,3
					sub fsize,3
				.elseif eax == 2 || eax == 3;ENCODING_UCS2BE and LE
					
					invoke DecodeDocument,lpBuffer,addr fsize,eax
					;//if the file's encoding is Unicode BE and LE,
					;//the operations of openning,including the SC_SETTEXT
					;//are executed in DecodeDocument in orded to use the 
					;//new buffer with the converted string in unicode
					
					mov lpBuffer,eax
					
					
				.endif
				invoke SetEncoding,_encoding
				SciMsg(SCI_CLEARALL)
				SciMsg(SCI_ADDTEXT,fsize,lpBuffer)
				invoke ChangeCaption,hWin,0
				
				invoke SysFreeString,lpBuffer
				
			
			.endif
				call SetFormat
				SciMsg(EM_SETSEL,0,0)
				SciMsg(SCI_SETCURRENTPOS,0)
ret
OpenFileName ENDP


SaveFileName PROC lpszFileName:DWORD
	
	LOCAL written :DWORD
	LOCAL txtsize :DWORD
	LOCAL lpBuffer :DWORD
	LOCAL hFile :DWORD
		
		;///encoding before save
		invoke EncodeDocument,_encoding

		lea ebx,szFileName
		cmp byte ptr[ebx],0
		jne @F
		xor eax,eax
		ret
		@@:
		SciMsg(SCI_GETTEXTLENGTH)
		mov txtsize,eax
		.if !txtsize ;//save an empty file
			invoke DeleteFile,addr szFileName
			invoke CreateFile,addr szFileName,GENERIC_WRITE,0,0,
				CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0
			mov hFile,eax
			invoke CloseHandle,hFile
			xor eax,eax
			ret
		.endif
		
		invoke CreateFile,addr szFileName,GENERIC_WRITE,0,0,
				CREATE_ALWAYS,0,0
		mov hFile,eax
		.if eax != INVALID_HANDLE_VALUE
			mov eax,txtsize
			add eax,1
			invoke SysAllocStringByteLen,0,eax
			mov lpBuffer,eax
			mov eax,txtsize
			add eax,1
			SciMsg(SCI_GETTEXT,eax,lpBuffer)
			
			;//encoding:
			.if _encoding > 1 ;//BIG or LITTLE Endian
				
				invoke EncodeToUnicode,lpBuffer,addr txtsize,_encoding
				mov lpBuffer,eax
			.endif
			
			invoke WriteFile,hFile,lpBuffer,txtsize,addr written,NULL
			invoke CloseHandle,hFile
			invoke SysFreeString,lpBuffer
			invoke ChangeCaption,hWin,2
		.endif
ret
SaveFileName ENDP